extern crate rlenc;

use std::fs::File;
use std::io::{Error, Read};

fn main() -> Result<(), Error> {
    let args: Vec<String> = std::env::args().collect();

    if args.len() != 2 {
        eprintln!("Invalid number of arguments. Usage: rlestat <input>");
        std::process::exit(1);
    }

    let mut file = File::open(&args[1])?;

    let mut data = Vec::new();
    file.read_to_end(&mut data)?;
    

    let analysis = rlenc::rle_analyse(&data);
    println!("Original size: {} bytes\nEstimated unescaped encoded size: {} bytes\n\
              Viable encoding escape byte: 0x{:02X} using {} bytes",
             analysis.original_size, analysis.encoded_unescaped_size_estimate,
             analysis.smallest_least_common_byte,
             analysis.count[analysis.smallest_least_common_byte as usize]);

    let encoded = rlenc::rle_encode(&data, None);
    println!("Unescaped encoded size: {} bytes", encoded.len());
    
    let encoded = rlenc::rle_encode(&data,
                                     Some(analysis.smallest_least_common_byte));
    println!("Escaped encoded size: {} bytes", encoded.len());

    Ok(())
}
