extern crate rlenc;

use std::fs::File;
use std::io::{Error, Read, Write};

fn main() -> Result<(), Error> {
    let args: Vec<String> = std::env::args().collect();

    if args.len() != 2 && args.len() != 3 {
        eprintln!("Invalid number of arguments. Usage: rlenc <input> [output]");
        std::process::exit(1);
    }

    if args.len() == 3 {
        let mut file = File::open(&args[1])?;
        let mut outfile = File::create(&args[2])?;

        let mut data = Vec::new();
        file.read_to_end(&mut data)?;
        let decoded = rlenc::rle_decode(&data,
                                         true);
        println!("Decoded size: {} bytes", decoded.len());
        
        outfile.write(&decoded)?;
    } else {
        let escape_byte = {
            let mut file = File::open(&args[1])?;

            let mut data = Vec::new();
            file.read_to_end(&mut data)?;
            let analysis = rlenc::rle_analyse(&data);
            
            println!("Original size: {} bytes\n\
                      Viable encoding escape byte: 0x{:02X} using {} bytes",
                     analysis.original_size, analysis.smallest_least_common_byte,
                     analysis.count[analysis.smallest_least_common_byte as usize]);
            analysis.smallest_least_common_byte
        };
        
        let mut file = File::open(&args[1])?;
        let mut outfile = File::create(&format!("{}.rle", args[1]))?;

        let mut data = Vec::new();
        file.read_to_end(&mut data)?;
        let encoded = rlenc::rle_encode(&data,
                                         Some(escape_byte));
        println!("Encoded size:  {} bytes", encoded.len());

        outfile.write(&encoded)?;
    }

    Ok(())
}
